import React, { useEffect, useState } from 'react';
import {
  useAppDispatch,
  useAuthorizationCache,
  useML,
  useQuerybuilderGraph,
  useQuerybuilderSettings,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { setCurrentTheme } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { resetGraphQueryResults, queryingBackend } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Navbar } from '../components/navbar/navbar';
import { Resizable } from '@graphpolaris/shared/lib/components/layout';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { EventBus } from '@graphpolaris/shared/lib/data-access/api/eventBus';
import { Onboarding } from '../components/onboarding/onboarding';
import { wsQueryRequest } from '@graphpolaris/shared/lib/data-access/broker';
import { URLParams, setParam } from '@graphpolaris/shared/lib/data-access/api/url';
import { VisualizationPanel } from '@graphpolaris/shared/lib/vis';
import { QueryBuilder } from '@graphpolaris/shared/lib/querybuilder';
import { SideNavTab, Sidebar } from '@graphpolaris/shared/lib/sidebar';
import { InspectorPanel } from '@graphpolaris/shared/lib/inspector';
import { SearchBar } from '@graphpolaris/shared/lib/sidebar/search/SearchBar';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';

export type App = {
  load?: string;
};

export function App(props: App) {
  const auth = useAuthorizationCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const ml = useML();
  const session = useSessionCache();
  const dispatch = useAppDispatch();
  const queryBuilderSettings = useQuerybuilderSettings();

  const runQuery = () => {
    if (session?.currentSaveState && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        dispatch(queryingBackend());
        wsQueryRequest(Query2BackendQuery(session.currentSaveState, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    if (props.load) {
      setParam(URLParams.saveState, props.load);
    }
  }, [props]);

  window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', event => {
    dispatch(setCurrentTheme(event.matches));
  });

  const [authCheck, setAuthCheck] = useState(false);
  const [tab, setTab] = useState<SideNavTab>('Schema');
  // const [visFullSize, setVisFullSize] = useState<boolean>(false);

  return (
    <div className="h-screen w-screen overflow-clip">
      <EventBus
        onRunQuery={runQuery}
        onAuthorized={() => {
          setAuthCheck(true);
        }}
      />
      {authCheck && (
        <>
          <Onboarding />
          <DashboardAlerts />
          <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
            <div className="flex flex-col h-screen max-h-screen relative">
              <aside className="absolute w-full h-12">
                <Navbar />
              </aside>
              <main className="grow flex flex-row h-screen pt-12">
                <Sidebar onTab={(tab) => setTab(tab)} tab={tab} />
                <Resizable divisorSize={3} horizontal={true} defaultProportion={0.85} maxProportion={0.85}>
                  <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
                    {tab !== undefined ? (
                      <div className="flex flex-col w-full h-full">
                        {tab === 'Search' && <SearchBar onRemove={() => setTab(undefined)} />}
                        {tab === 'Schema' && <Schema auth={authCheck} onRemove={() => setTab(undefined)} />}
                      </div>
                    ) : null}

                    <Resizable divisorSize={3} horizontal={false}>
                      <VisualizationPanel
                        fullSize={() => {
                          // setVisFullSize(!visFullSize);
                          // tab === undefined && setTab('Schema');
                          // tab !== undefined && setTab(undefined);
                        }}
                      />

                      <QueryBuilder onRunQuery={runQuery} />
                    </Resizable>
                  </Resizable>
                  <InspectorPanel />
                </Resizable>
              </main>
            </div>
          </div>
        </>
      )}
    </div>
  );
}

export default App;
